@@ -41,6 +41,19 @@
 import unittest, subprocess, re, pexpect, smtplib, socket, os, time, tempfile
 import testlib
 
+import sys
+''' Test for postfix check output'''
+result = subprocess.run(['postconf', 'maillog_file = /dev/stdout'], capture_output=True, text=True)
+result = subprocess.run(['postfix', 'check'], capture_output=True, text=True)
+if result.returncode != 0:
+    print('postfix check failed with error code: {0}.'.format(result.returncode))
+    sys.exit(1)
+if result.stdout:
+    print('postfix check warning/error: {0}'.format(result.stdout[26:]))
+    subprocess.run(['postconf', 'maillog_file ='], capture_output=True, text=True)
+    sys.exit(1)
+subprocess.run(['postconf', 'maillog_file ='], capture_output=True, text=True)
+
 class PostfixTest(testlib.TestlibCase):
     '''Test Postfix MTA.'''
 
@@ -49,13 +62,13 @@ class PostfixTest(testlib.TestlibCase):
 
         # Move listener to localhost:2525
         conf_file = '/etc/postfix/master.cf'
-        lines = open(conf_file)
         contents = ''
-        for cfline in lines:
-            if cfline.startswith('smtp') and 'smtpd' in cfline and 'inet' in cfline:
-                contents += '127.0.0.1:2525      inet  n       -       -       -       -       smtpd\n'
-            else:
-                contents += "%s\n" % cfline
+        with open(conf_file) as fh:
+            for cfline in fh:
+                if cfline.startswith('smtp') and 'smtpd' in cfline and 'inet' in cfline:
+                    contents += '127.0.0.1:2525      inet  n       -       -       -       -       smtpd\n'
+                else:
+                    contents += "%s\n" % cfline
         testlib.config_replace(conf_file, contents, append=False)
 
         conf_file = '/etc/postfix/main.cf'
@@ -96,6 +109,7 @@ class PostfixTest(testlib.TestlibCase):
         subprocess.call(['mkdir','-p','/var/run/saslauthd'])
         subprocess.call(['/usr/sbin/service', 'saslauthd', 'stop'], stdout=subprocess.PIPE)
         subprocess.call(['/usr/sbin/service', 'saslauthd', 'start'], stdout=subprocess.PIPE)
+        subprocess.call(['cp', '/etc/sasldb2', '/var/spool/postfix/etc/sasldb2'])
 
     def tearDown(self):
         '''Clean up after each test_* function'''
@@ -143,12 +157,12 @@ mech_list: %s %s
             # attach SASL to postfix chroot
             subprocess.call(['mkdir','-p','/var/spool/postfix/var/run/saslauthd'])
             subprocess.call(['rm','-rf','/var/run/saslauthd'])
-            subprocess.call(['ln','-s','/var/spool/postfix/var/run/saslauthd','/var/run/saslauthd'])
+            subprocess.call(['ln','-s','/var/run/saslauthd', '/var/spool/postfix/var/run/saslauthd'])
             subprocess.call(['/usr/sbin/service', 'saslauthd', 'stop'], stdout=subprocess.PIPE)
             assert subprocess.call(['/usr/sbin/service', 'saslauthd', 'start'], stdout=subprocess.PIPE, stderr=subprocess.STDOUT) == 0
 
             # Force crackful perms so chroot'd postfix can talk to saslauthd
-            subprocess.call(['chmod','o+x','/var/spool/postfix/var/run/saslauthd'])
+            subprocess.call(['chmod','o+x','/var/run/saslauthd'])
         else:
             plaintext = "1"
             if mech == "LOGIN" or mech == "PLAIN":
@@ -166,7 +180,7 @@ mech_list: %s %s
             rc, report = testlib.cmd(['postconf', '-h', 'myhostname'])
             expected = 0
             result = 'Got exit code %d, expected %d\n' % (rc, expected)
-            self.assertEquals(expected, rc, result + report)
+            self.assertEqual(expected, rc, result + report)
 
             child = pexpect.spawn('saslpasswd2 -c -u %s %s' % (report.strip(), self.user.login))
             time.sleep(0.2)
@@ -180,15 +194,15 @@ mech_list: %s %s
             time.sleep(0.2)
             rc = child.expect('\n', timeout=5)
             time.sleep(0.2)
-            self.assertEquals(rc, expected, "passwd returned %d" %(rc))
+            self.assertEqual(rc, expected, "passwd returned %d" %(rc))
 
             child.kill(0)
 
-            os.chmod("/etc/sasldb2", 0640)
+            os.chmod("/etc/sasldb2", 0o640)
             rc, report = testlib.cmd(['chgrp', 'postfix', '/etc/sasldb2'])
             expected = 0
             result = 'Got exit code %d, expected %d\n' % (rc, expected)
-            self.assertEquals(expected, rc, result + report)
+            self.assertEqual(expected, rc, result + report)
 
             # Force crackful perms so chroot'd postfix can talk to saslauthd
             subprocess.call(['mv', '-f', '/etc/sasldb2', '/var/spool/postfix/etc'])
@@ -205,7 +219,7 @@ mech_list: %s %s
         s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         s.settimeout(5)
         s.connect(('localhost',2525))
-        greeting = s.recv(1024)
+        greeting = s.recv(1024).decode('utf-8')
         # 220 gorgon.outflux.net ESMTP Postfix (Ubuntu)
         self.assertTrue(greeting.startswith('220 '),greeting)
         self.assertTrue('ESMTP' in greeting,greeting)
@@ -225,12 +239,12 @@ mech_list: %s %s
         code, msg = self.s.getreply()
         reply = '%d %s' % (code, msg)
         if valid:
-            self.assertEquals(code, 252, reply)
-            self.assertTrue(address in msg, reply)
+            self.assertEqual(code, 252, reply)
+            self.assertTrue(address.encode('utf-8') in msg, reply)
         else:
-            self.assertEquals(code, 550, reply)
-            self.assertTrue('Recipient address rejected' in msg, reply)
-            self.assertTrue('<%s>' % (address) in msg, reply)
+            self.assertEqual(code, 550, reply)
+            self.assertTrue(b'Recipient address rejected' in msg, reply)
+            self.assertTrue(('<%s>' % (address)).encode('utf-8') in msg, reply)
 
     def test_10_commands(self):
         '''Basic SMTP commands'''
@@ -239,15 +253,15 @@ mech_list: %s %s
         # EHLO
         code, msg = self.s.ehlo()
         reply = '%d %s' % (code, msg)
-        self.assertEquals(code, 250, reply)
-        self.assertEquals(self.s.does_esmtp, 1, reply)
-        self.assertTrue('8BITMIME' in self.s.ehlo_resp, reply)
-        # No help available
+        self.assertEqual(code, 250, reply)
+        self.assertEqual(self.s.does_esmtp, 1, reply)
+        self.assertTrue(b'8BITMIME' in self.s.ehlo_resp, reply)
+        # Help available as of postfix 3.9
         self.s.putcmd("help")
         code, msg = self.s.getreply()
         reply = '%d %s' % (code, msg)
-        self.assertEquals(code, 502, reply)
-        self.assertTrue('Error' in msg, reply)
+        self.assertEqual(code, 214, reply)
+        self.assertTrue(b'2.0.0 Commands:' in msg, reply)
         # VRFY addresses
         self._vrfy('address@example.com', valid=True)
         self._vrfy('does-not-exist', valid=False)
@@ -268,9 +282,9 @@ Hello, nice to meet you.
 ''' % (user_sent_to.gecos, user_sent_to.login))
         #for addr in failed.keys():
         #    print '%s %d %s' % (addr, failed[addr][0], failed[addr][1])
-        self.assertEquals(len(failed),1,failed)
-        self.assertTrue(failed.has_key('does-not-exist'),failed)
-        self.assertEquals(failed['does-not-exist'][0],550,failed)
+        self.assertEqual(len(failed),1,failed)
+        self.assertTrue('does-not-exist' in failed,failed)
+        self.assertEqual(failed['does-not-exist'][0],550,failed)
 
         # Frighteningly, postfix seems to accept email before confirming
         # a successful write to disk for the recipient!
@@ -286,7 +300,8 @@ Hello, nice to meet you.
         if spool_file == None:
             spool_file = '/var/mail/%s' % (target_spool_user.login)
         time.sleep(1)
-        contents = open(spool_file).read()
+        with open(spool_file) as fh:
+            contents = fh.read()
         # Server-side added headers...
         self.assertTrue('\nReceived: ' in contents, contents)
         if use_tls and self.lsb_release['Release'] > 6.06:
@@ -335,7 +350,8 @@ Hello, nice to meet you.
 
     def _write_forward(self, user, contents):
         forward_filename = '/home/%s/.forward' % (user.login)
-        open(forward_filename,'w').write(contents)
+        with open(forward_filename,'w') as fh:
+            fh.write(contents)
         os.chown(forward_filename, user.uid, user.gid)
 
     def test_10_sending_mail_forward_normal(self):
@@ -358,7 +374,7 @@ Hello, nice to meet you.
 /bin/cat > "%s"
 ''' % (mbox_name),prefix='test-postfix.redir-')
         redir.close()
-        os.chmod(redir_name,0755)
+        os.chmod(redir_name,0o755)
     
         self._write_forward(self.user,'|%s\n' % (redir_name))
 
@@ -373,15 +389,16 @@ Hello, nice to meet you.
 
         # First, create our "target" file
         secret = '/root/secret.txt'
-        open(secret,'w').write('Secret information\n')
-        os.chmod(secret, 0700)
+        with open(secret,'w') as fh:
+            fh.write('Secret information\n')
+        os.chmod(secret, 0o700)
 
         # Now, create a symlink to the target (we're going to use /var/tmp
         # since we're assuming it, /root, /var/mail are on the same filesystem.
         # For most chroot testing, /tmp is mounted from the real machine.
-        if os.path.exists('/var/tmp/secret.link'):
+        if os.path.islink('/var/tmp/secret.link'):
             os.unlink('/var/tmp/secret.link')
-        self.assertEquals(subprocess.call(['su','-c','ln -s /root/secret.txt /var/tmp/secret.link',self.user.login]),0,"Symlink creation")
+        self.assertEqual(subprocess.call(['su','-c','ln -s /root/secret.txt /var/tmp/secret.link',self.user.login]),0,"Symlink creation")
 
         # Now, the hardlink, which in ubuntu's case needs to be done by root.
         os.link('/var/tmp/secret.link','/var/mail/%s' % (self.user.login))
@@ -394,12 +411,13 @@ Subject: This is an overwrite test
 
 Hello, nice to pwn you.
 ''' % (self.user.gecos, self.user.login))
-        self.assertEquals(len(failed),0,failed)
+        self.assertEqual(len(failed),0,failed)
 
         # Pause for delivery
         time.sleep(2)
 
-        contents = open(secret).read()
+        with open(secret) as fh:
+            contents = fh.read()
         # Clean up before possible failures
         os.unlink('/var/mail/%s' % (self.user.login))
         os.unlink('/var/tmp/secret.link')
@@ -421,9 +439,9 @@ Hello, nice to pwn you.
         # has mech
         code, msg = self.s.ehlo()
         reply = '%d %s' % (code, msg)
-        self.assertEquals(code, 250, reply)
-        self.assertEquals(self.s.does_esmtp, 1, reply)
-        self.assertTrue('%s' % mech in self.s.ehlo_resp, reply)
+        self.assertEqual(code, 250, reply)
+        self.assertEqual(self.s.does_esmtp, 1, reply)
+        self.assertTrue(mech.encode('utf-8') in self.s.ehlo_resp, reply)
         return reply
 
     def test_20_sasldb_cram_md5(self):
@@ -496,14 +514,14 @@ Hello, nice to pwn you.
         rc, report = testlib.cmd(['postconf', 'smtpd_sasl_auth_enable'])
         expected = 0
         result = 'Got exit code %d, expected %d\n' % (rc, expected)
-        self.assertEquals(expected, rc, result + report)
+        self.assertEqual(expected, rc, result + report)
         self.assertTrue('yes' in report, "Could not find 'yes' in report:\n%s" % report)
 
         if self.lsb_release['Release'] > 6.06:
             rc, report = testlib.cmd(['postconf', 'smtpd_sasl_type'])
             expected = 0
             result = 'Got exit code %d, expected %d\n' % (rc, expected)
-            self.assertEquals(expected, rc, result + report)
+            self.assertEqual(expected, rc, result + report)
             self.assertTrue('cyrus' in report, "Could not find 'cyrus' in report:\n%s" % report)
 
         # ehlo
@@ -512,11 +530,11 @@ Hello, nice to pwn you.
 
         code, msg = self.s.docmd("AUTH", "CRAM-MD5")
         reply = '%d %s' % (code, msg)
-        self.assertEquals(code, 334, reply)
+        self.assertEqual(code, 334, reply)
 
         code, msg = self.s.docmd("*")
         reply = '%d %s' % (code, msg)
-        self.assertEquals(code, 501, reply)
+        self.assertEqual(code, 501, reply)
 
         error = False
         try:
@@ -525,7 +543,7 @@ Hello, nice to pwn you.
             error = True
         self.assertFalse(error, "server disconnected")
         reply = '%d %s' % (code, msg)
-        self.assertEquals(code, 334, reply)
+        self.assertEqual(code, 334, reply)
 
     def test_99_restore(self):
         '''Restore configuration'''
